use pom::parser::*;
use pom::parser::Parser;
use std::fs::File;
use std::io::Read;

fn crlf<'a>() -> Parser<'a, u8, ()> {
    one_of(b"\r\n").repeat(0..).discard()
}
fn main() {
	let mut file = File::open("assets/data.json").unwrap();
	let mut input = Vec::new();
	file.read_to_end(&mut input).unwrap();

    let outputOk = crlf().parse("test".as_bytes());
    let outputErr = crlf().parse(&input);
}
